/*
 *   $Id: linklayer.c,v 1.1 1996/11/12 23:20:50 lf Exp $
 *
 *   Authors:
 *    Lars Fenneberg		<lf@elemental.net>	 
 *
 *   This software is Copyright 1996 by the above mentioned author(s), 
 *   All Rights Reserved.
 *
 *   The license which is distributed with this software in the file COPYRIGHT
 *   applies to this software. If your distribution is missing this file, you
 *   may request it from <lf@elemental.net>.
 *
 */

#include <sys/ioctl.h>
#include <syslog.h>
#include <linux/if_arp.h>

#include "radv.h"
#include "defaults.h"

/*
 * this list maps hardware types to lengths (in bits) of the link layer
 * addresses
 */

struct LLTokenLength {
	int ifhwtype;
	int tokenlen;
	int (*fillin_hwaddr)(struct Interface *, void *);
};

int ether_fillin(struct Interface *, void *);

static struct LLTokenLength HWTtoLLTL[] = {
	{ARPHRD_ARCNET	,	 8,		NULL},
	{ARPHRD_ETHER	,	48,	ether_fillin},
#ifdef ARPHRD_FDDI
	{ARPHRD_FDDI	,	48,	ether_fillin},
#endif
	{-1, -1, NULL}
};

/*
 * this function gets the hardware type and address of an interface,
 * determines the link layer token length and checks it against
 * the defined prefixes
 */
int
setup_linklayer_addr(int sock, struct Interface *iface)
{
	int i;
	struct ifreq	ifr;
	struct AdvPrefix *prefix;
	
	strcpy(ifr.ifr_name, iface->Name);
	
	if (ioctl(sock, SIOCGIFHWADDR, &ifr) < 0) {
		log(LOG_ERR, "ioctl(SIOCGIFHWADDR) failed for %s", iface->Name);
		return (-1);
	}

	dlog(LOG_DEBUG, 3, "hardware type for %s is %d", iface->Name,
		ifr.ifr_hwaddr.sa_family); 
	
	memcpy(&iface->if_hwaddr, &ifr.ifr_hwaddr, sizeof(struct sockaddr));
	
	for(i=0; HWTtoLLTL[i].ifhwtype != -1; i++)
		if (iface->if_hwaddr.sa_family == HWTtoLLTL[i].ifhwtype)
			break;
		
	dlog(LOG_DEBUG, 3, "link layer token length for %s is %d", iface->Name, 
		HWTtoLLTL[i].tokenlen);
		
	iface->tokenlen = HWTtoLLTL[i].tokenlen;
	iface->fillin_hwaddr = HWTtoLLTL[i].fillin_hwaddr;

	if (iface->tokenlen != -1) {
		prefix = iface->AdvPrefixList;
		while (prefix) {
			if ((iface->tokenlen + prefix->PrefixLen) 
					!= MAX_PrefixLen) {
				
				log(LOG_WARNING, "prefix length should be %d"
					" for %s", MAX_PrefixLen - iface->tokenlen,
					iface->Name);
			}
			prefix = prefix->next;
		}
	} else {
		log(LOG_WARNING, "unknown interface type %s, can't determine"
			" lenght of link layer token", iface->Name);
	}
		
	return (0);
}

/*
 * this function fills in an ethernet hardware address.
 * (return_value + 2) must always be a multiple of eight.
 */

int
ether_fillin(struct Interface *iface, void *buff)
{
	memcpy(buff, iface->if_hwaddr.sa_data, 6);
	return 6;
}
