/*
 *   $Id: process.c,v 1.1 1996/11/12 23:20:50 lf Exp $
 *
 *   Authors:
 *    Pedro Roque		<roque@di.fc.ul.pt>
 *    Lars Fenneberg		<lf@elemental.net>	 
 *
 *   This software is Copyright 1996 by the above mentioned author(s), 
 *   All Rights Reserved.
 *
 *   The license which is distributed with this software in the file COPYRIGHT
 *   applies to this software. If your distribution is missing this file, you
 *   may request it from <lf@elemental.net>.
 *
 */

#include <syslog.h>
#include <string.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netinet6/in6.h>
#include <netinet6/icmpv6.h>
#include <net/ndisc.h>
#include <net/addrconf.h>

#include "radv.h"

void
process_msg(int sock, struct Interface *ifacel, char *msg, int len, 
	struct sockaddr_in6 *addr, struct in6_pktinfo *pkt_info)
{
	struct Interface *iface;
	struct icmpv6hdr *icmph;

	icmph = (struct icmpv6hdr *) msg;

	if (icmph->type != NDISC_ROUTER_SOLICITATION &&
	    icmph->type != NDISC_ROUTER_ADVERTISEMENT)
	{
		/*
		 *	We just want to listen to RSs and RAs
		 */
		
		dlog(LOG_DEBUG, 2, "icmp filter failed");
		return;
	}

	dlog(LOG_DEBUG, 4, "if_index %d", pkt_info->ipi6_ifindex);

	/* get iface by receive if_index */

	for (iface = ifacel; iface; iface=iface->next)
	{
		if (iface->if_index == pkt_info->ipi6_ifindex)
		{
			break;
		}
	}

	if (iface == NULL)
	{
		log(LOG_WARNING, "recveived packet for unknown interface\n");
		return;
	}

	dlog(LOG_DEBUG, 4, "found Interface: %s", iface->Name);

	if (icmph->type == NDISC_ROUTER_SOLICITATION)
	{
		struct timeval tv;

		gettimeofday(&tv, NULL);
		
		if (tv.tv_sec - iface->tstamp < iface->MinRtrAdvInterval)
		{
			/*
			 *	unicast reply
			 */
			send_ra(sock, iface, &addr->sin6_addr);
		}
		else
		{
			/*
			 *	multicast reply
			 */

			clear_timer(&iface->tm);
			send_ra(sock, iface, NULL);
			set_timer(&iface->tm, iface->MaxRtrAdvInterval);
		}
	}
}
