/*
 *   $Id: radv.h,v 1.1 1996/11/12 23:20:50 lf Exp $
 *
 *   Authors:
 *    Pedro Roque		<roque@di.fc.ul.pt>
 *    Lars Fenneberg		<lf@elemental.net>	 
 *
 *   This software is Copyright 1996 by the above mentioned author(s), 
 *   All Rights Reserved.
 *
 *   The license which is distributed with this software in the file COPYRIGHT
 *   applies to this software. If your distribution is missing this file, you
 *   may request it from <lf@elemental.net>.
 *
 */

#ifndef RADV_H
#define RADV_H

#include <netinet6/in6.h>
#include <netinet6/ipv6.h>
#include <net/if.h>

/* for log.c */
#define	L_NONE		0
#define L_SYSLOG	1
#define L_STDERR	2
#define L_LOGFILE	3

struct timer_lst {
	unsigned long		expires;
	void			(*handler)(void *);
	void *			data;
	struct timer_lst	*next;
	struct timer_lst	*prev;	
};

struct AdvPrefix;

struct Interface {
	char			Name[IFNAMSIZ];		/* interface name */

	struct in6_addr		if_addr;
	int			if_index;

	int			tokenlen;
	struct sockaddr		if_hwaddr;
	int 			(*fillin_hwaddr)(struct Interface *, void *);

	int			AdvSendAdvert;
	int			MaxRtrAdvInterval;
	int			MinRtrAdvInterval;
	int			AdvManagedFlag;
	int			AdvOtherConfigFlag;
	int			AdvLinkMTU;
	int			AdvReachableTime;
	int			AdvRetransTimer;
	int			AdvCurHopLimit;
	int			AdvDefaultLifetime;
	int			AdvSourceLLAddress;

	struct AdvPrefix	*AdvPrefixList;
	struct timer_lst	tm;
	unsigned long		tstamp;
	struct Interface	*next;
};

struct AdvPrefix {
	struct in6_addr		Prefix;
	int			PrefixLen;
	
	int			AdvOnLinkFlag;
	int			AdvAutonomousFlag;
	__u32			AdvValidLifetime;
	__u32			AdvPreferedLifetime;

	struct AdvPrefix	*next;
};

/* gram.y */
int yyparse(void);

/* scanner.l */
int yylex(void);

/* timer.c */
extern void set_timer(struct timer_lst *tm, int secs);
extern void clear_timer(struct timer_lst *tm);

/* log.c */
int log_open(int, char *, char*);
int log(int, char *, ...);
int dlog(int, int, char *, ...);
int log_close(void);
void set_debuglevel(int);
int get_debuglevel(void);

/* linklayer.c */
int setup_linklayer_addr(int sock, struct Interface *);

/* linklocal.c */
int setup_linklocal_addr(struct Interface *);

/* socket.c */
int open_icmpv6_socket(void);

/* send.c */
void send_ra(int, struct Interface *iface, struct in6_addr *dest);

/* process.c */
void process_msg(int sock, struct Interface *, char *, int,
	struct sockaddr_in6 *, struct in6_pktinfo *);

/* recv.c */
int recv_rs(int, char *, struct sockaddr_in6 *, struct in6_pktinfo **);

#endif
