/*
 *   $Id: send.c,v 1.1 1996/11/12 23:20:50 lf Exp $
 *
 *   Authors:
 *    Pedro Roque		<roque@di.fc.ul.pt>
 *    Lars Fenneberg		<lf@elemental.net>	 
 *
 *   This software is Copyright 1996 by the above mentioned author(s), 
 *   All Rights Reserved.
 *
 *   The license which is distributed with this software in the file COPYRIGHT
 *   applies to this software. If your distribution is missing this file, you
 *   may request it from <lf@elemental.net>.
 *
 */

#include <syslog.h>
#include <string.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netinet6/in6.h>
#include <netinet6/icmpv6.h>
#include <net/ndisc.h>
#include <net/addrconf.h>

#include "radv.h"

void
send_ra(int sock, struct Interface *iface, struct in6_addr *dest)
{
	const struct in6_addr all_hosts_addr = {{{
		0xFF, 0x02, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 1
	}}};
	struct sockaddr_in6 addr;
	struct in6_pktinfo *pkt_info;
	struct msghdr mhdr;
	struct cmsghdr *cmsg;
	struct iovec iov;
	char chdr[sizeof(struct cmsghdr) + sizeof(struct in6_pktinfo)];

	struct icmpv6hdr *icmph;
	struct AdvPrefix *prefix;
	unsigned char buff[2048];
	int len = 0;
	int err;
	__u8  *ucp;
	__u32 *ulp;


	dlog(LOG_DEBUG, 3, "sending RA on %s", iface->Name);

	if (dest == NULL)
	{
		struct timeval tv;

		dest = &all_hosts_addr;
		gettimeofday(&tv, NULL);
		iface->tstamp = tv.tv_sec;
	}
	
	addr.sin6_family = AF_INET6;
	addr.sin6_port = htons(IPPROTO_ICMPV6);
	memcpy(&addr.sin6_addr, dest, sizeof(struct in6_addr));


	icmph = (struct icmpv6hdr *) buff;

	icmph->type = NDISC_ROUTER_ADVERTISEMENT;
	icmph->code = 0;
	icmph->checksum = 0;

	icmph->icmp6_hop_limit		= iface->AdvCurHopLimit;
	icmph->icmp6_addrconf_managed	= iface->AdvManagedFlag;
	icmph->icmp6_addrconf_other	= iface->AdvOtherConfigFlag;
	icmph->icmp6_rt_lifetime	= iface->AdvDefaultLifetime;

	len = sizeof(struct icmpv6hdr);

	ulp = (__u32 *) (buff + len);

	*ulp = htonl(iface->AdvReachableTime);
	len += sizeof(__u32);
	ulp++;

	*ulp = htonl(iface->AdvRetransTimer);
	len += sizeof(__u32);

	prefix = iface->AdvPrefixList;

	/*
	 *	add prefix options
	 */

	while(prefix)
	{
		struct prefix_info *pinfo;
		
		pinfo = (struct prefix_info *) (buff + len);

		pinfo->type		= ND_OPT_PREFIX_INFO;
		pinfo->length		= 4;
		pinfo->prefix_len	= prefix->PrefixLen;
		pinfo->onlink		= prefix->AdvOnLinkFlag;
		pinfo->autoconf		= prefix->AdvAutonomousFlag;
		pinfo->reserved		= 0;
		pinfo->valid		= htonl(prefix->AdvValidLifetime);
		pinfo->prefered		= htonl(prefix->AdvPreferedLifetime);
		pinfo->reserved2	= 0;
		
		memcpy(&pinfo->prefix, &prefix->Prefix,
		       sizeof(struct in6_addr));

		len += sizeof(struct prefix_info);

		prefix = prefix->next;
	}
	
	/*
	 *	add MTU option
	 */

	if (iface->AdvLinkMTU != 0) {
		ucp = (__u8 *)	(buff + len);
	
		/* type (1 octet), length (1 octet), last bits are reserved */
		*ucp++ = ND_OPT_MTU;
		*ucp++ = 1;
		*ucp++ = 0; 
		*ucp++ = 0;
		len += 4 * sizeof(__u8);

		ulp = (__u32 *)	(buff + len);

		*ulp = htonl(iface->AdvLinkMTU);
		len += sizeof(__u32);
	}

	/*
	 * add Source Link-layer Address option
	 */

	if (iface->AdvSourceLLAddress && iface->fillin_hwaddr) {
		int hwaddr_len;
	
		ucp = (__u8 *)	(buff + len);
	
		/* type (1 octet), length (1 octet), last bits are reserved */
		*ucp++ = ND_OPT_SOURCE_LL_ADDR;
		/* the "+ 63" rounds up */ 
		*ucp++ = (__u8) ((iface->tokenlen + 16 + 63) / 64);
		len += 2 * sizeof(__u8);

		hwaddr_len = iface->fillin_hwaddr(iface, (void *)ucp);

		len += hwaddr_len;
		ucp += hwaddr_len;
	}

	iov.iov_len  = len;
	iov.iov_base = buff;
	
	cmsg = (struct cmsghdr *) chdr;
	
	cmsg->cmsg_len   = (sizeof(struct cmsghdr) +
			    sizeof(struct in6_pktinfo));
	cmsg->cmsg_level = SOL_IPV6;
	cmsg->cmsg_type  = IPV6_TXINFO;
	
	pkt_info = (struct in6_pktinfo *) cmsg->cmsg_data;
	memset(pkt_info, 0, sizeof(struct in6_pktinfo));
	pkt_info->ipi6_ifindex = iface->if_index;
	memcpy(&pkt_info->ipi6_addr, &iface->if_addr, sizeof(struct in6_addr));

	mhdr.msg_name = &addr;
	mhdr.msg_namelen = sizeof(struct sockaddr_in6);
	mhdr.msg_iov = &iov;
	mhdr.msg_iovlen = 1;
	mhdr.msg_control = (void *) cmsg;
	mhdr.msg_controllen = (sizeof(struct cmsghdr) +
			       sizeof(struct in6_pktinfo));

	err = sendmsg(sock, &mhdr, 0);
	
	if (err < 0) {
		log(LOG_WARNING, "sendmsg: %s", strerror(errno));
	}
}
